/**************************************************************************************/
/*    Demo program for:                                         */
/*	  Board: Shield-MIDI + PIC32-PINGUINO-OTG_MX220              */
/*    Manufacture: OLIMEX                                       */
/*	  COPYRIGHT (C) 2013							                               */
/*    Designed by:      Penko T. Bozhkov                        */
/*    Module Name    :  Demo_Example                            */
/*    File   Name    :  Shield_MIDI_Demo_Pinguino-OTG_MX220.pde */
/*    Revision       :  1.0.0 (initial)                         */
/*    Date           :  23.05.2013                              */
/*    Built with:                                               */
/*      Pinguino-X4-EasyPack-Win32-Rev685mod-v01-2013-02        */
/**************************************************************************************/
/*
  Description:
  ============
  When PIC32-PINGUINO-OTG/MX220's BUT is pressed, then its LED2(Yellow/Red) is switched over and stream of messages is sent over the MIDI-OUT connector.
  This stream is part of a song. During this process, receiving is not possible in the current demo example.
  Once the sending has been finished, receiving is enabled.
  Whenever message is received, PIC32-PINGUINO-OTG/MX220's LED1(Green) is switched over.
  The received data might be displayed on the software serial terminal if one day such a library is written. :-)
  The physical pins that are recommended to refer to this terminal are: D12 for RxD, D11 for TxD and GND must not be forgotten!
  At PIC32-PINGUINO-OTG/MX220 board, this pins are connected to the UEXT connector. This gives us an opportunity to connect
  there USB-Serial-Cable-F, see https://www.olimex.com/Products/Components/Cables/USB-Serial-Cable/USB-Serial-Cable-F/ 
  and use it as a terminal for observing all received MIDI messages. The required physical connections are as follows:
  USB-Serial-Cable-F(Blue)  -> connected to UEXT pin 2 (GND)
  USB-Serial-Cable-F(Red)   -> connected to UEXT pin 7 (D12, i.e. RxD)
  USB-Serial-Cable-F(Green) -> connected to UEXT pin 8 (D11, i.e. TxD)
  Once you have done this connections, plug your USB-Serial-Cable-F in any USB and open serial port with settings: 57600 8N1
  and enjoy while you are improvising your ideas! :-)
  Other idea is to use CDC instead for observing console. As much as PIC32-PINGUINO-MX220 is concerned, there is no enough free memory space.
  Trials can be done with PIC32-PINGUINO-OTG, because it has got enough free memory space. Although it is true, till this moment my trials are not successful(See the comments)!
  If any is able to have a go with this issue, please share with me your skills! ;-) I'll be grateful!
*/

#include <interrupt.c>
//#include <__cdc.c>

//extern void putUSBUSART(char *data, char Length);

#if defined(PIC32_PINGUINO_220)
    const int buttonPin = 8;            // the number of the pushbutton pin
    const int ledPin1 =  13;            // the number of the LED1 pin
    const int ledPin2 =  9;             // the number of the LED2 pin
#elif defined(PIC32_PINGUINO_OTG)
    const int buttonPin = 2;            // the number of the pushbutton pin
    const int ledPin1 =  13;            // the number of the LED1 pin
    //const int ledPin2 =  9;           // the number of the LED2 -> not assigned in this board
    #define LED2_Output     TRISD &= ~0x02; ODCD &= ~0x02; 
    #define LED2_Toggle     PORTD ^= 0x02;
#else
    #error: "The selected board is not supported!!!" 
#endif
    
#define NumberDetections 50         // Number of the detections before accepting Button as pressed
// MIDI Related
#define Note_On     0x90            // Note On at channel 0
#define Note_Off    0x80            // Note Off at channel 0
#define Velocity    0x50            // Velocity 80
#define Note_A3     0x39            // This is the note A from C3 - B3: small octave     
#define Note_B3     0x3B            // This is the note B from C3 - B3: small octave     
#define Note_C4     0x3C            // This is the note C from C4 - B4: one-line octave, i.e. C4: middle C
#define Note_D4     0x3E            // This is the note D from C4 - B4: one-line octave
#define Note_E4     0x40            // This is the note E from C4 - B4: one-line octave
#define Note_F4     0x41            // This is the note F from C4 - B4: one-line octave
// variables will change:
int buttonState = 0;         // variable for reading the pushbutton status
int buttonCounter = NumberDetections;
int MIDIRecBytes = 0;
unsigned char temp = 0;


void MIDI_Message(unsigned char CMD, unsigned char Data, unsigned char Vel) __attribute__ ((noinline));
void MIDI_Message(unsigned char CMD, unsigned char Data, unsigned char Vel)
{
    Serial.write(CMD);
    Serial.write(Data);
    Serial.write(Vel);
}

void setup()  
{
  noInterrupts();
  
  // initialize the LED pins as outputs:
  pinMode(ledPin1, OUTPUT);
  #if defined(PIC32_PINGUINO_220)
    pinMode(ledPin2, OUTPUT);
  #elif defined(PIC32_PINGUINO_OTG)
    LED2_Output;
  #endif
  // initialize the pushbutton pin as an input:
  pinMode(buttonPin, INPUT); 
  // Open serial communications and wait for port to open:
  Serial.begin(31250);  // Select the MIDI standard baud rate
  
  interrupts();
  
/*  
  // set the data rate for the SoftwareSerial port(must be higher than the baudrate of the MIDI communication)
  CDC.printf("\f"\
             "=====================\n\r"\
             " MIDI Demo Program!!!\n\r"\
             "=====================\n\r"\
             "Received Data in HEX format: \n\r");
*/
  
}

void loop() // run over and over
{
  // Scan the button
  buttonState = digitalRead(buttonPin);

    // Check if really pressed?
  if (buttonState == LOW) {
    // The button is detected as pressed
    buttonCounter--;
    if(buttonCounter==0){
      // The button is really pressed!
      // Switch over the led
      #if defined(PIC32_PINGUINO_220)
        if(digitalRead(ledPin2)==HIGH){  digitalWrite(ledPin2, LOW);  }         // turn LED off
        else{                           digitalWrite(ledPin2, HIGH); }         // turn LED on
      #elif defined(PIC32_PINGUINO_OTG)
        LED2_Toggle;
      #endif
      
      // Send MIDI Messages to the music synthesizer
      // Note A3
      MIDI_Message(Note_On,  Note_A3, Velocity);
      delay(250);
      MIDI_Message(Note_Off, Note_A3, Velocity);
      delay(250);
      // Notes A3 and B3
      MIDI_Message(Note_On,  Note_A3, Velocity);
      MIDI_Message(Note_On,  Note_B3, Velocity);
      delay(250);
      MIDI_Message(Note_Off, Note_A3, Velocity);
      MIDI_Message(Note_Off, Note_B3, Velocity);
      delay(250);
      // Notes A3 and C4
      MIDI_Message(Note_On,  Note_A3, Velocity);
      MIDI_Message(Note_On,  Note_C4, Velocity);
      delay(250);
      MIDI_Message(Note_Off, Note_A3, Velocity);
      MIDI_Message(Note_Off, Note_C4, Velocity);
      delay(250);
      // Notes C4 and E4
      MIDI_Message(Note_On,  Note_C4, Velocity);
      MIDI_Message(Note_On,  Note_E4, Velocity);
      delay(250);
      MIDI_Message(Note_Off, Note_C4, Velocity);
      MIDI_Message(Note_Off, Note_E4, Velocity);
      delay(250);
      // Notes D4 and F4
      MIDI_Message(Note_On,  Note_D4, Velocity);
      MIDI_Message(Note_On,  Note_F4, Velocity);
      delay(375);
      MIDI_Message(Note_Off, Note_D4, Velocity);
      MIDI_Message(Note_Off, Note_F4, Velocity);
      delay(250);
      // Notes C4 and E4
      MIDI_Message(Note_On,  Note_C4, Velocity);
      MIDI_Message(Note_On,  Note_E4, Velocity);
      delay(125);
      MIDI_Message(Note_Off, Note_C4, Velocity);
      MIDI_Message(Note_Off, Note_E4, Velocity);
      delay(250);
      // Notes C4 and E4
      MIDI_Message(Note_On,  Note_C4, Velocity);
      MIDI_Message(Note_On,  Note_E4, Velocity);
      delay(750);
      MIDI_Message(Note_Off, Note_C4, Velocity);
      MIDI_Message(Note_Off, Note_E4, Velocity);
      delay(750);
      // Notes B3 and D4
      MIDI_Message(Note_On,  Note_B3, Velocity);
      MIDI_Message(Note_On,  Note_D4, Velocity);
      delay(750);
      MIDI_Message(Note_Off, Note_B3, Velocity);
      MIDI_Message(Note_Off, Note_D4, Velocity);
      delay(750);
     
      // Wait button to be released
      buttonCounter = NumberDetections;
      while(buttonCounter){
        buttonCounter--;
        while(digitalRead(buttonPin)==LOW){  }  // Wait button to released
      }
      buttonCounter = NumberDetections;         // Prepare the button counter for the next detection
    }
  }
  else{ buttonCounter = NumberDetections; }  // Reload the counter because error detection has occured! 
  
  
  // If any MIDI messages are incomming, display them to the mySerial terminal
  while(Serial.available()){
    temp = Serial.read();
/*    
    // HEX to ASCII converter
    if((temp>>4)>9){ putUSBUSART(((temp>>4)+0x37),1); }      // 0x37+0x0A = 0x41 (ASCII of "A")
    else{ putUSBUSART(((temp>>4)+0x30),1); }                 // 0x30 (ASCII of "0")
    // Send ASCII code of the first digit
    if((temp&0x0F)>9){ putUSBUSART(((temp&0x0F)+0x37),1); }  // 0x37+0x0A = 0x41 (ASCII of "A")
    else{ putUSBUSART(((temp&0x0F)+0x30),1); }               // 0x30 (ASCII of "0")
    // Send ASCII code of the second digit
    //
    // Print only 6 HEX digits at one row
    MIDIRecBytes++;
    //if(!(MIDIRecBytes&1)){ mySerial.write(0x20); } // Space if even number
    putUSBUSART(0x20,1); // Space
    if(MIDIRecBytes==6){
      MIDIRecBytes = 0;
      //CDC.printf("\n\r");  // New line
    }
*/    
    
    // Switch over LED1 at every received byte
    if(digitalRead(ledPin1)==HIGH){  digitalWrite(ledPin1, LOW);  }         // turn LED off
    else{                           digitalWrite(ledPin1, HIGH); }         // turn LED on
    //
  }
    
}


 
